use std::sync::Arc;

use serenity::async_trait;
use serenity::client::bridge::gateway::GatewayIntents;
use serenity::http::AttachmentType;
use serenity::model::prelude::{Message, Ready};
use serenity::prelude::*;
use structopt::StructOpt;

use discord::commands::command_framework;
use discord::helper::send_msg_to_player_channels;
use game::global_data::GlobalData;
use game::MessageSource;

use crate::config::{Args, BotConfig};

mod config;
mod discord;
mod error;
mod game;

struct Handler {}

#[async_trait]
impl EventHandler for Handler {
    async fn message(&self, ctx: Context, msg: Message) {
        if msg.author.bot {
            return;
        }

        if msg.content.starts_with('$') {
            return;
        }

        let data = ctx.data.read().await;

        let global_data = data.get::<GlobalData>().unwrap();

        let mut global_data = global_data.lock().await;

        if global_data.game_state.is_none() {
            // no game in progress
            return;
        }

        if let Some(player_data) = global_data
            .game_state()
            .unwrap()
            .get_player_from_channel(msg.channel_id.0)
        {
            let guild = msg.guild(&ctx.cache).await.unwrap();
            let user_msg = format!("**{}** > {}", player_data.codename, msg.content);

            let attachments: Vec<AttachmentType> = msg
                .attachments
                .iter()
                .map(|a| AttachmentType::Image(&a.url))
                .collect();

            send_msg_to_player_channels(
                &ctx,
                &guild,
                &mut global_data,
                MessageSource::Player(msg.channel_id.0),
                &user_msg,
                Some(attachments),
                false,
            )
            .await
            .expect("Unable to send message to players");
        }
    }

    async fn ready(&self, _ctx: Context, ready: Ready) {
        println!("{} is connected!", ready.user.name);
    }
}

#[tokio::main]
async fn main() {
    let args: Args = Args::from_args();

    let bot_cfg: BotConfig = BotConfig::new(&args.cfg_path).expect("Unable to parse cfg");

    let mut global_data = GlobalData::new(bot_cfg.clone());

    if global_data.game_state_exists() {
        println!("Resuming game...");
        global_data
            .load_game_state()
            .expect("Unable to open saved game state.");
    }

    let mut client = Client::builder(&bot_cfg.token)
        .event_handler(Handler {})
        .framework(command_framework())
        .intents(GatewayIntents::all())
        .type_map_insert::<GlobalData>(Arc::new(Mutex::new(global_data)))
        .await
        .expect("Err creating client");

    if let Err(why) = client.start().await {
        println!("Client error: {:?}", why);
    }
}
